package javax.sound.sampled;

import java.util.Collections;
import java.util.Map;
import java.util.HashMap;

public class AudioFileFormat
{
    private AudioFileFormat.Type type;
    private int byteLength;
    private AudioFormat format;
    private int frameLength;
    private HashMap<String, Object> properties;
    
    protected AudioFileFormat(AudioFileFormat.Type type, int byteLength, AudioFormat format, int frameLength) {
        this.type = type;
        this.byteLength = byteLength;
        this.format = format;
        this.frameLength = frameLength;
        this.properties = null;
    }
    
    public AudioFileFormat(AudioFileFormat.Type type, AudioFormat audioFormat, int n) {
        this(type, -1, audioFormat, n);
    }
    
    public AudioFileFormat(AudioFileFormat.Type type, AudioFormat audioFormat, int n, Map<String, Object> m) {
        this(type, -1, audioFormat, n);
        this.properties = new HashMap(m);
    }
    
    public AudioFileFormat.Type getType() {
        return this.type;
    }
    
    public int getByteLength() {
        return this.byteLength;
    }
    
    public AudioFormat getFormat() {
        return this.format;
    }
    
    public int getFrameLength() {
        return this.frameLength;
    }
    
    public Map<String, Object> properties() {
        HashMap<Object, Object> m;
        if (this.properties == null) {
            m = new HashMap(0);
        }
        else {
            m = (Map)(Map)this.properties.clone();
        }
        return Collections.unmodifiableMap(m);
    }
    
    public Object getProperty(String key) {
        if (this.properties == null) {
            return null;
        }
        return this.properties.get(key);
    }
    
    @Override
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        if (this.type != null) {
            buffer.append(this.type.toString() + " (." + this.type.getExtension() + ") file");
        }
        else {
            buffer.append("unknown file format");
        }
        if (this.byteLength != -1) {
            buffer.append(", byte length: " + this.byteLength);
        }
        buffer.append(", data format: " + this.format);
        if (this.frameLength != -1) {
            buffer.append(", frame length: " + this.frameLength);
        }
        return new String(buffer);
    }
    
    public static class Type
    {
        public static final AudioFileFormat.Type WAVE;
        public static final AudioFileFormat.Type AU;
        public static final AudioFileFormat.Type AIFF;
        public static final AudioFileFormat.Type AIFC;
        public static final AudioFileFormat.Type SND;
        private final String name;
        private final String extension;
        
        public Type(String name, String extension) {
            this.name = name;
            this.extension = extension;
        }
        
        @Override
        public final boolean equals(Object o) {
            if (this.toString() == null) {
                return o != null && o.toString() == null;
            }
            return o instanceof AudioFileFormat.Type && this.toString().equals(o.toString());
        }
        
        @Override
        public final int hashCode() {
            if (this.toString() == null) {
                return 0;
            }
            return this.toString().hashCode();
        }
        
        @Override
        public final String toString() {
            return this.name;
        }
        
        public String getExtension() {
            return this.extension;
        }
        
        static {
            WAVE = new AudioFileFormat.Type("WAVE", "wav");
            AU = new AudioFileFormat.Type("AU", "au");
            AIFF = new AudioFileFormat.Type("AIFF", "aif");
            AIFC = new AudioFileFormat.Type("AIFF-C", "aifc");
            SND = new AudioFileFormat.Type("SND", "snd");
        }
    }
}
